package de.unihalle.informatik.Alida.grappa;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.imageio.ImageIO;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.KeyStroke;

import de.unihalle.informatik.Alida.annotations.ALDAOperator;
import de.unihalle.informatik.Alida.gui.ALDChooseOpNameFrame;
import de.unihalle.informatik.Alida.gui.ALDOperatorChooserTree;
import de.unihalle.informatik.Alida.gui.OnlineHelpDisplayer;

/**
 * Menu bar for Grappa window.
 * 
 * @author Birgit Moeller
 */

public class ALDGrappaMenuBar extends JMenuBar implements ActionListener {
	
	/**
	 * Reference to main window of grappa.
	 */
	protected ALDGrappaFrame grappaMainWin;
	
	/**
	 * Reference to the operator chooser tree of window this menubar belongs to.
	 */
	protected ALDOperatorChooserTree opTree;
	
	/**
	 * Reference to the workbench of window this menubar belongs to.
	 */
	protected ALDGrappaWorkbench workBench;
	
	/**
	 * Icon to be shown in about box.
	 */
	protected ImageIcon aboutIcon;

	/**
	 * Default constructor
	 */
	public ALDGrappaMenuBar(ALDGrappaFrame _grappaMainWin, 
			ALDOperatorChooserTree _opTree,	ALDGrappaWorkbench _workBench) {
		super();
		this.grappaMainWin = _grappaMainWin;
		this.opTree = _opTree;
		this.workBench = _workBench;
		// init the icon for the about box
		this.setupAboutIcon();
		// init the menu
		this.setupMenu();
	}
	
	/**
	 * Generates the menubar.
	 */
	protected void setupMenu() {
		// file menue
		JMenu fileM = new JMenu("File");
		JMenuItem quitItem = new JMenuItem("Quit");
		quitItem.setActionCommand("quit");
		quitItem.addActionListener(this);
		fileM.add(quitItem);

		// operator level menue
		JMenu opSelectM = new JMenu("Operator Level");
		ButtonGroup operatorLevelGroup = new ButtonGroup();
		JRadioButtonMenuItem radioItemApplication = new JRadioButtonMenuItem(
				"Application");
		radioItemApplication.setActionCommand("viewApps");
		radioItemApplication.addActionListener(this);
		JRadioButtonMenuItem radioItemStandard = new JRadioButtonMenuItem(
				"Standard");
		radioItemStandard.setActionCommand("viewStd");
		radioItemStandard.addActionListener(this);
		if (this.opTree.getLevel().equals(ALDAOperator.Level.APPLICATION)) 
			radioItemApplication.setSelected(true);
		else
			radioItemStandard.setSelected(true);
		operatorLevelGroup.add(radioItemApplication);
		operatorLevelGroup.add(radioItemStandard);
		opSelectM.add(radioItemApplication);
		opSelectM.add(radioItemStandard);

//		ButtonGroup operatorLevelGroup = new ButtonGroup();
//		JRadioButtonMenuItem radioItemApplication = new JRadioButtonMenuItem(
//				"Application");
//		radioItemApplication.setActionCommand("viewApps");
//		radioItemApplication.addActionListener(this);
//		JRadioButtonMenuItem radioItemStandard = new JRadioButtonMenuItem(
//				"Standard");
//		radioItemStandard.setActionCommand("viewStd");
//		radioItemStandard.addActionListener(this);
//		if (this.opTree.getLevel().equals(ALDAOperator.Level.APPLICATION)) 
//			radioItemApplication.setSelected(true);
//		else
//			radioItemStandard.setSelected(true);
//		operatorLevelGroup.add(radioItemApplication);
//		operatorLevelGroup.add(radioItemStandard);
//		opFilterM.add(radioItemApplication);
//		opFilterM.add(radioItemStandard);

		// workflow menu
		JMenu actionsM = new JMenu("Workflow");
		JMenuItem newItem = new JMenuItem("New");
		newItem.setActionCommand("new");
		newItem.addActionListener(this);
		newItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		JMenuItem closeItem = new JMenuItem("Close");
		closeItem.setActionCommand("close");
		closeItem.addActionListener(this);
		closeItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_W, ActionEvent.CTRL_MASK));
		JMenuItem loadItem = new JMenuItem("Load");
		loadItem.setActionCommand("load");
		loadItem.addActionListener(this);
		loadItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_L, ActionEvent.CTRL_MASK));
		JMenuItem saveItem = new JMenuItem("Save");
		saveItem.setActionCommand("save");
		saveItem.addActionListener(this);
		saveItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		JMenuItem renameItem = new JMenuItem("Rename");
		renameItem.setActionCommand("rename");
		renameItem.addActionListener(this);
		renameItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_U, ActionEvent.CTRL_MASK));
		JMenuItem runItem = new JMenuItem("Run");
		runItem.setActionCommand("run");
		runItem.addActionListener(this);
		runItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_A, ActionEvent.CTRL_MASK));
		JMenuItem stopItem = new JMenuItem("Stop");
		stopItem.setActionCommand("stop");
		stopItem.addActionListener(this);
		newItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		actionsM.add(newItem);
		actionsM.add(renameItem);
		actionsM.add(closeItem);
		actionsM.addSeparator();
		actionsM.add(loadItem);
		actionsM.add(saveItem);
		actionsM.addSeparator();
		actionsM.add(runItem);
		actionsM.add(stopItem);

		// help menu
		JMenu helpM = this.generateHelpMenu();
		
		// add to menu bar and finally to the main window
		this.add(fileM);
		this.add(opSelectM);
		this.add(actionsM);
		this.add(Box.createHorizontalGlue());
		this.add(helpM);
	}

	/**
	 * Method to initialize the Grappa icon in the about box.
	 */
	protected void setupAboutIcon() {
		String iconDataName = "/share/logo/Grappa_logo.png";
		Image img = null;
		BufferedImage bi = null;
		Graphics g = null;
		InputStream is = null;
		try {
			ImageIcon icon;
			File iconDataFile = new File("./" + iconDataName);
			if(iconDataFile.exists()) {
				icon = new ImageIcon("./" + iconDataName);
				img = icon.getImage();
			}
			// try to find it inside a jar archive....
			else {
				is = ALDChooseOpNameFrame.class.getResourceAsStream(iconDataName);
				if (is == null) {
					System.err.println("Warning - cannot find icons...");
					img = new BufferedImage(20,20,BufferedImage.TYPE_INT_ARGB);
				}
				else {
					img = ImageIO.read(is);
				}
				bi= new BufferedImage(20,20,BufferedImage.TYPE_INT_ARGB);
				g = bi.createGraphics();
				g.drawImage(img, 0, 0, 20, 20, null);
			}
		} catch (IOException ex) {
			System.err.println("ALDGrappaMenuBar - problems loading icons...!");
			img = new BufferedImage(20,20,BufferedImage.TYPE_INT_ARGB);
			bi= new BufferedImage(20,20,BufferedImage.TYPE_INT_ARGB);
			g = bi.createGraphics();
			g.drawImage(img, 0, 0, 20, 20, null);
		}
		this.aboutIcon = new ImageIcon(img);
	}

	/**
	 * Function to setup the help menu.
	 * @return	Help menu.
	 */
	protected JMenu generateHelpMenu() {
		JMenu helpM = new JMenu("Help");
		JMenuItem itemHelp = new JMenuItem("Online Help");
		itemHelp.addActionListener(
				OnlineHelpDisplayer.getHelpActionListener(itemHelp,"welcome"));
		JMenuItem itemAbout = new JMenuItem("About Grappa");
		itemAbout.setActionCommand("showAbout");
		itemAbout.addActionListener(this);
		helpM.add(itemHelp);
		helpM.add(itemAbout);
		return helpM;
	}	

	@Override
	public void actionPerformed(ActionEvent ev) {
		// extract command
		String command = ev.getActionCommand();

		// handle tree view options
		if (command.equals("viewApps")) {
			this.opTree.setLevel(ALDAOperator.Level.APPLICATION);
		} 
		else if (command.equals("viewStd")) {
			this.opTree.setLevel(ALDAOperator.Level.STANDARD);
		}
		else if (command.equals("new")) {
			this.workBench.addNewWorkflow();
		}
		else if (command.equals("close")) {
			this.workBench.removeWorkflow();
		}
		else if (command.equals("save")) {
			this.workBench.saveWorkflow();
		}
		else if (command.equals("load")) {
			this.workBench.loadWorkflow();
		}
		else if (command.equals("run")) {
			this.workBench.runWorkflow();
		}
		else if (command.equals("run")) {
			this.workBench.interruptWorkflowExecution();
		}
		else if (command.equals("quit")) {
			this.grappaMainWin.quit();
		}
		else if (command.equals("rename")) {
			this.workBench.renameWorkflow();
		}
		else if (command.equals("showAbout")) {
			this.showAboutBox();
		}
	}

	/**
	 * Show an about box window.
	 * <p>
	 * Method is supposed to be overwritten by subclasses.
	 */
	protected void showAboutBox() {
		Object[] options = { "OK" };
		String msg = 
				"<html>Grappa - The Graphical Program Editor for Alida<p><p>"
		   + "\u00a9 2010 - 2013   "
		   + "Martin Luther University Halle-Wittenberg<p><p>"
		   + "Institute of Computer Science, Faculty of Natural Sciences III<p><p>"
		   + "Email: alida@informatik.uni-halle.de <p>"
		   + "Internet: <i>www.informatik.uni-halle.de/alida</i><p>"
		   + "License: GPL 3.0, <i>http://www.gnu.org/licenses/gpl.html</i></html>";

		JOptionPane.showOptionDialog(null, new JLabel(msg),
		    "Information about Grappa", JOptionPane.DEFAULT_OPTION,
		    JOptionPane.INFORMATION_MESSAGE, this.aboutIcon, options, options[0]);
	}

}
