/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2013
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev$
 * $Date$
 * $Author$
 * 
 */

package de.unihalle.informatik.Alida.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.concurrent.BlockingDeque;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import de.unihalle.informatik.Alida.batch.ALDBatchRunResultInfo;
import de.unihalle.informatik.Alida.batch.ALDBatchOutputManagerSwing.ProviderInteractionLevel;
import de.unihalle.informatik.Alida.dataio.ALDDataIOManagerSwing;
import de.unihalle.informatik.Alida.exceptions.ALDBatchIOException;
import de.unihalle.informatik.Alida.exceptions.ALDOperatorException;
import de.unihalle.informatik.Alida.exceptions.ALDWorkflowException;
import de.unihalle.informatik.Alida.gui.ALDOperatorResultFrame;
import de.unihalle.informatik.Alida.operator.ALDOperator;
import de.unihalle.informatik.Alida.operator.ALDOperatorControllable;
import de.unihalle.informatik.Alida.operator.ALDOperatorLocation;
import de.unihalle.informatik.Alida.operator.events.*;
import de.unihalle.informatik.Alida.operator.events.ALDControlEvent.ALDControlEventType;
import de.unihalle.informatik.Alida.workflows.ALDWorkflow;
import de.unihalle.informatik.Alida.workflows.ALDWorkflow.ALDWorkflowContextType;
import de.unihalle.informatik.Alida.workflows.ALDWorkflowNodeID;
import de.unihalle.informatik.Alida.workflows.ALDWorkflowNode.ALDWorkflowNodeState;
import de.unihalle.informatik.Alida.workflows.events.ALDWorkflowEvent;
import de.unihalle.informatik.Alida.workflows.events.ALDWorkflowRunFailureInfo;
import de.unihalle.informatik.Alida.workflows.events.ALDWorkflowEvent.*;
import de.unihalle.informatik.Alida.workflows.events.ALDWorkflowEventListener;

/**
 * Manager for executing single operator and workflow objects via GUI.
 * 
 * @author Birgit Moeller
 */
public class ALDOperatorGUIExecutionProxy 
	implements ALDWorkflowEventListener, ActionListener {
	
	/* ***********************************
	 * Some local data type declarations.
	 * ***********************************/
	
	/**
	 * Status of the associated thread to execute operator.
	 */
	protected static enum WorkflowThreadStatus {
		/**
		 * Thread is ready for execution.
		 */
		THREAD_READY,
		/**
		 * Thread is running.
		 */
		THREAD_RUNNING,
		/**
		 * Thread is paused.
		 */
		THREAD_PAUSED,
		/**
		 * Thread was interrupted.
		 */
		THREAD_INTERRUPTED,
		/**
		 * Thread is not running.
		 */
		THREAD_STOPPED
	}

	/**
	 * Labels to be used on buttons of Yes/No message boxes.
	 */
	protected final Object[] yesnoOption = { "YES", "NO" };

	/* ******************
	 * Internal members.
	 * ******************/

	/**
	 * Reference to the underlying Alida workflow object.
	 */
	protected ALDWorkflow alidaWorkflow;

	/**
	 * Current status of the operator execution thread.
	 */
	protected WorkflowThreadStatus workflowStatus = WorkflowThreadStatus.THREAD_READY;

	/**
	 * Corresponding configuration and control window.
	 */
	protected ALDOperatorControlFrame controlWin;

	/**
	 * Corresponding configuration and control window.
	 */
	protected JFrame failureMessageWin;

	/**
	 * Listener object attached to the control window.
	 */
	protected ParameterChangeListener paramChangeListener;
	
	/**
	 * Reference ID of the operator node in Alida workflow;
	 */
	protected ALDWorkflowNodeID operatorNodeID;
	
	/**
	 * Frame showing operator execution results.
	 */
	protected ALDOperatorResultFrame resultFrame;
	
	/**
	 * Message window shown while workflow execution is aborted.
	 */
//	protected JFrame interruptWin;

	/**
	 * Flag to indicate if the is an abortion currently going on.
	 */
	protected boolean processEvents = true;
	
//	/**
//	 * Flag to indicate if a node is initially added.
//	 * <p>
//	 * If so, windows and other things need to be initialized, if not,
//	 * they just need to be updated.
//	 */
//	protected boolean initialInit = true;
	
	/**
	 * Default constructor.
	 */
	public ALDOperatorGUIExecutionProxy(ALDOperatorLocation opLocation) {
		try {
			this.alidaWorkflow = 
					new ALDWorkflow(" ", ALDWorkflowContextType.OP_RUNNER);
//			this.alidaWorkflow.setNotifyRecursiveFlag(true);
			this.alidaWorkflow.addALDWorkflowEventListener(this);
		} catch (ALDOperatorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// init the operator and its workflow, i.e. add a single node to the flow
		this.initOperatorWorkflow(opLocation);
		
		// some additional initializations
		this.initInterruptWin();
	}
	
	/**
	 * Setup of the window for renaming workflows.
	 */
	protected void initInterruptWin() {
//		this.processEvents = true;
//		this.interruptWin = new JFrame("Message");
//		JPanel mainPanel = new JPanel();
//		mainPanel.add(new JLabel("Please wait, aborting execution..."));
//		this.interruptWin.add(mainPanel);
//		this.interruptWin.setSize(300,115);
//		this.interruptWin.setResizable(false);
	}

	/**
	 * Adds the operator as node to the workflow.
	 * @param opPath	Full path of operator object to be handled by this manager.
	 */
	protected void initOperatorWorkflow(ALDOperatorLocation opLocation) {
		this.createNewWorkflowNode(opLocation);
	}
	
	/**
	 * Creates a new workflow node.
	 * @param loc 	Location from where to instantiate the operator.
	 */
	protected void createNewWorkflowNode(ALDOperatorLocation loc) {
		try {
			this.operatorNodeID = this.alidaWorkflow.createNode(loc);
		} catch (ALDWorkflowException ex) {
			JOptionPane.showMessageDialog(null, "Instantiation of operator \""
					+ loc.getName() + "\" failed!\n", 
					"Error", JOptionPane.ERROR_MESSAGE);
		}
		// process workflow events
		this.processWorkflowEventQueue();
	}

	/**
	 * Displays the configuration and control window.
	 */
	public void showGUI() {
		this.controlWin.setVisible(true);
	}
	
	public void configureWorkflow(ALDConfigurationEvent confEvent) {
		this.alidaWorkflow.handleALDConfigurationEvent(confEvent);
		// post-process workflow events
		this.processWorkflowEventQueue();
	}

	/**
	 * Method called by GUI if a new operator was loaded.
	 * <p>
	 * This method updates the workflow by deleting the old operator node and
	 * adding a new one for the new operator. Note that the GUI components are
	 * not newly initialized, but are only updated. This is triggered by setting
	 * the variable <code>initialInit</code> to false.
	 * 
	 * @param newOp		New operator loaded from file.
	 * @throws ALDWorkflowException
	 */
	public void updateOperator(ALDOperator newOp) throws ALDWorkflowException {
//		this.initialInit = false;
//		// update workflow, remove old node and add new one
//		this.alidaWorkflow.removeNode(this.operatorNodeID);
//		this.operatorNodeID = this.alidaWorkflow.createNode(newOp);
		this.alidaWorkflow.setOperator(this.operatorNodeID, newOp);
		// process workflow events
		this.processWorkflowEventQueue();
	}
	
	/**
	 * Returns current status of workflow thread.
	 * @return	Status of workflow thread.
	 */
	public WorkflowThreadStatus getWorkflowThreadStatus() {
		return this.workflowStatus;
	}
	
	/**
	 * Executes the workflow.
	 */
	public void runWorkflow() {
		try {
			this.processEvents = true;
			ALDWorkflowNodeState state =
					this.alidaWorkflow.getState(this.operatorNodeID);
			if (   !this.workflowStatus.equals(WorkflowThreadStatus.THREAD_STOPPED)
					&& state.equals(ALDWorkflowNodeState.READY)) {
				if ( this.alidaWorkflow.getOperator(this.operatorNodeID) 
									instanceof	ALDOperatorControllable) {
					if (!this.workflowStatus.equals(
							WorkflowThreadStatus.THREAD_INTERRUPTED)) {
						JOptionPane.showMessageDialog(null, 
								"Operator was already executed with current configuration,\n" +
								"please change parameters before executing it again.",
								"Alida Message", JOptionPane.INFORMATION_MESSAGE);
						return;
//						int answer = JOptionPane.showConfirmDialog(null, 
//								"Operator was run with current configuration already!\n" +
//										"Do you really want to exectue it again?",
//										"Workflow Message", JOptionPane.YES_NO_OPTION, 
//										JOptionPane.WARNING_MESSAGE);
//						if (answer == JOptionPane.NO_OPTION) {
//							return;
//						}
					}
					this.resultFrame = null;
					this.alidaWorkflow.handleALDControlEvent(
							new ALDControlEvent(this, ALDControlEventType.RUN_EVENT));
					this.alidaWorkflow.nodeParameterChanged(this.operatorNodeID);
					this.workflowStatus = WorkflowThreadStatus.THREAD_RUNNING;
					this.alidaWorkflow.runWorkflow();
				}
				else {
					// non-controllables operators thould not be run again with same config
					JOptionPane.showMessageDialog(null, 
							"Operator was already executed with current configuration,\n" +
							"please change parameters before executing it again.",
							"Alida Message", JOptionPane.INFORMATION_MESSAGE);
					return;
				}
			}
			else {
				this.resultFrame = null;
				this.workflowStatus = WorkflowThreadStatus.THREAD_RUNNING;
				this.alidaWorkflow.runWorkflow();
			}
		} catch (ALDWorkflowException e) {
			JOptionPane.showMessageDialog(null, "Executing operator failed!\n",
					"Error", JOptionPane.ERROR_MESSAGE);
		}
		// post-process workflow events
		this.processWorkflowEventQueue();
	}
	
	protected boolean batchModeActive;
	
	protected String batchInputParameter;
	
	protected Iterator<Object> batchInputIterator;
	
	protected LinkedList<String> batchOutputParameter;
	
	protected HashMap<String, ALDBatchRunResultInfo> batchOutputResultMap;
	
	public void runWorkflowInBatchMode() 
			throws ALDBatchIOException, ALDOperatorException {
		LinkedList<String> batchParams = this.controlWin.getBatchInputParameters();
		// TODO I am not sure whether that's the way to go (sp)
		if ( batchParams.size() > 0) {
			this.batchInputParameter = batchParams.get(0);
			this.batchInputIterator = 
					this.controlWin.getInputParamIterator(this.batchInputParameter);
			this.batchOutputParameter = this.controlWin.getBatchOutputParameters();
			this.batchOutputResultMap = new HashMap<String, ALDBatchRunResultInfo>();
			for (String output : this.batchOutputParameter) {
				this.batchOutputResultMap.put(output, new ALDBatchRunResultInfo(output));
			}
			this.batchModeActive = true;
			this.workflowStatus = WorkflowThreadStatus.THREAD_RUNNING;
			this.doNextBatchModeStep();
		}
	}
	
	protected void doNextBatchModeStep() {
		if (this.batchInputIterator.hasNext()) {
			Object nextItem = this.batchInputIterator.next();
			for (String output : this.batchOutputParameter) {
				this.batchOutputResultMap.get(output).
																				getParameterValueVec().add(nextItem);
			}
			try {
				this.alidaWorkflow.getOperator(this.operatorNodeID).setParameter(
																					this.batchInputParameter, nextItem);
				this.alidaWorkflow.nodeParameterChanged(this.operatorNodeID);
				this.processWorkflowEventQueue();
				this.alidaWorkflow.getOperator(this.operatorNodeID).print();
				this.alidaWorkflow.runWorkflow();
			} catch (ALDWorkflowException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ALDOperatorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		else {
			ALDBatchOperatorResultFrame resultFrame;
			try {
				resultFrame = new ALDBatchOperatorResultFrame(
						this.alidaWorkflow.getOperator(this.operatorNodeID),
						this.batchOutputResultMap, 
						ProviderInteractionLevel.WARNINGS_ONLY);
				resultFrame.setVisible(true);
				this.batchModeActive = false;
			} catch (ALDWorkflowException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void pauseWorkflow() {
		this.workflowStatus = WorkflowThreadStatus.THREAD_PAUSED;
		this.alidaWorkflow.handleALDControlEvent(
				new ALDControlEvent(this, ALDControlEventType.PAUSE_EVENT));
		// post-process workflow events
		this.processWorkflowEventQueue();
	}
	
	public void resumeWorkflow() {
		this.workflowStatus = WorkflowThreadStatus.THREAD_RUNNING;
		this.alidaWorkflow.handleALDControlEvent(
				new ALDControlEvent(this, ALDControlEventType.RESUME_EVENT));
		// post-process workflow events
		this.processWorkflowEventQueue();
	}
	
	public void doNextStepInWorkflow() {
		this.workflowStatus = WorkflowThreadStatus.THREAD_RUNNING;
		this.alidaWorkflow.handleALDControlEvent(
				new ALDControlEvent(this, ALDControlEventType.STEP_EVENT));
		// post-process workflow events
		this.processWorkflowEventQueue();
	}
	
	public void stopWorkflow() {
		this.workflowStatus = WorkflowThreadStatus.THREAD_STOPPED;
		this.alidaWorkflow.handleALDControlEvent(
				new ALDControlEvent(this, ALDControlEventType.STOP_EVENT));
		// post-process workflow events
		this.processWorkflowEventQueue();
	}

	// TODO Kill event vs. interrupt Execution!
	public void killWorkflowThread() {
//		this.alidaWorkflow.handleALDControlEvent(
//				new ALDControlEvent(this, ALDControlEventType.KILL_EVENT));
		this.processEvents = false;
//		this.interruptWin.setVisible(true);
		this.alidaWorkflow.interruptExecution();
		// post-process workflow events
		this.processWorkflowEventQueue();
	}
	
	/**
	 * Aborts running execution of the workflow.
	 */
	public void interruptExecution() {
		this.processEvents = false;
//		this.interruptWin.setVisible(true);
		this.alidaWorkflow.interruptExecution();
		// post-process workflow events
		this.processWorkflowEventQueue();
	}
	
	/**
	 * (Re-)display the result frame again, if results available.
	 */
	public void showResultFrame() {
		if (this.resultFrame != null)
			this.resultFrame.setVisible(true);
	}
	
	/**
	 * Does clean-up on termination, i.e. closes all open windows.
	 */
	public boolean quit() {
		this.processEvents = false;
		// check if thread terminated
		this.processWorkflowEventQueue();
		if (   this.workflowStatus == WorkflowThreadStatus.THREAD_RUNNING
				|| this.workflowStatus == WorkflowThreadStatus.THREAD_PAUSED) {
			int selection = JOptionPane
					.showOptionDialog(
							null,
							"The operator is running or paused!\n"
									+ "Do you really want to quit?",
									"Warning", JOptionPane.DEFAULT_OPTION,
									JOptionPane.WARNING_MESSAGE, null,
									this.yesnoOption, this.yesnoOption[0]);
			switch (selection) 
			{
			case JOptionPane.YES_OPTION:
//				this.execProxy.killWorkflowThread();
				this.workflowStatus = WorkflowThreadStatus.THREAD_STOPPED;
				//	((ALDOperatorControllable) this.op)
				//	.handleALDControlEvent(new ALDControlEvent(this,
				//			ALDControlEventType.STOP_EVENT));
				break;
			case JOptionPane.NO_OPTION:
			case JOptionPane.CANCEL_OPTION:
				return false;
			}
		} 
		else {
			this.workflowStatus = WorkflowThreadStatus.THREAD_STOPPED;
		}
		// try to close the configuration window
		return this.controlWin.quit();
	}
	
	/**
	 * Processes all events that were recently added to the queue.
	 * <p>
	 * Note that this function needs to be called after all actions on the 
	 * Alida workflow except calls to 'run' methods.
	 */
	protected synchronized void processWorkflowEventQueue() {
		BlockingDeque<ALDWorkflowEvent> queue = 
				this.alidaWorkflow.getEventQueue(this);
		ALDWorkflowEvent event = null; 	
		while (!queue.isEmpty()) {
			event = queue.pop();
			this.handleALDWorkflowEvent(event);
		}
	}
	
	/* (non-Javadoc)
	 * @see de.unihalle.informatik.Alida.workflows.events.ALDWorkflowEventListener#handleALDWorkflowEvent(de.unihalle.informatik.Alida.workflows.events.ALDWorkflowEvent)
	 */
	@SuppressWarnings("unchecked")
  @Override
	public synchronized void handleALDWorkflowEvent(ALDWorkflowEvent event) {

		// do not process anymore any events if termination is going on...
		if (!this.processEvents) {
			return;
		}

		// extract event data
		ALDWorkflowEventType type = event.getEventType();
		Object eventInfo = event.getId();

		// handle the event
		switch(type) 
		{
		case ADD_NODE:
			if (eventInfo instanceof ALDWorkflowNodeID) {
				ALDOperator op;
				try {
					op = this.alidaWorkflow.getOperator((ALDWorkflowNodeID)eventInfo);
					this.handleAddNodeEvent(op, (ALDWorkflowNodeID)eventInfo);
				} catch (ALDWorkflowException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				this.workflowStatus = WorkflowThreadStatus.THREAD_READY;
			}
			break;
		case NODE_PARAMETER_CHANGE:
			this.handleNodeParameterChangeEvent(
																(Collection<ALDWorkflowNodeID>)eventInfo);
			break;
		case NODE_STATE_CHANGE:
			this.handleNodeStateChangeEvent(
																(Collection<ALDWorkflowNodeID>)eventInfo);
			break;
		case RUN_FAILURE:
			this.workflowStatus = WorkflowThreadStatus.THREAD_READY;
			this.displayFailureMessageWindow(event);
			break;
		case USER_INTERRUPT:
//			JOptionPane.showMessageDialog(null, "Execution was aborted!", 
//					"Workflow Execution Message", JOptionPane.ERROR_MESSAGE);
			this.workflowStatus = WorkflowThreadStatus.THREAD_INTERRUPTED;
			break;
		case EXECUTION_FINISHED:
			if (this.workflowStatus.equals(WorkflowThreadStatus.THREAD_STOPPED))
				this.workflowStatus = WorkflowThreadStatus.THREAD_STOPPED;
			// inform the control window of the termination of the thread
			this.controlWin.updateNodeStatus(ALDWorkflowNodeState.READY);
			break;
		case SHOW_RESULTS:
			if (!(eventInfo instanceof ALDWorkflowNodeID)) {
				JOptionPane.showMessageDialog(null, "Cannot display results!", 
						"Workflow Execution Error", JOptionPane.ERROR_MESSAGE);
				return;
			}
			if (this.batchModeActive) {
				ALDOperator op;
				try {
					op = this.alidaWorkflow.getOperator((ALDWorkflowNodeID)eventInfo);
					for (String output : this.batchOutputParameter) {
						Object resultValue = op.getParameter(output);
						this.batchOutputResultMap.get(output).
																					getResultDataVec().add(resultValue);
					}
				} catch (ALDWorkflowException e) {
					JOptionPane.showMessageDialog(null, "Cannot extract results!", 
							"Workflow Execution Error", JOptionPane.ERROR_MESSAGE);
				} catch (ALDOperatorException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				this.doNextBatchModeStep();
			}
			else {
				// there are results to display
				ALDOperator op;
				try {
					op = this.alidaWorkflow.getOperator((ALDWorkflowNodeID)eventInfo);
					this.resultFrame = new ALDOperatorResultFrame(op, 
									ALDDataIOManagerSwing.ProviderInteractionLevel.WARNINGS_ONLY);
					this.resultFrame.setVisible(true);
				} catch (ALDWorkflowException e) {
					JOptionPane.showMessageDialog(null, "Cannot display results!", 
							"Workflow Execution Error", JOptionPane.ERROR_MESSAGE);
				}
				this.workflowStatus = WorkflowThreadStatus.THREAD_READY;
			}
			break;
		case RENAME:
			// event is specific for Grappa editor, ignored here
			break;
		default:
			System.out.println("Event type \'" + type + "\' not yet handled...");
		}
	}

	@Override
  public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command.equals("closeFailWin")) {
			this.failureMessageWin.setVisible(false);
		}
  }

	/**
	 * Displays window with detailed error information on execution failures.
	 * @param event		Workflow event got on failure.
	 */
	protected void displayFailureMessageWindow(ALDWorkflowEvent event) {
		// extract event data
		Object eventInfo = event.getId();

		// eventInfo object is of type ALDWorkflowRunFailureInfo...
		if (eventInfo instanceof ALDWorkflowRunFailureInfo) {
			ALDWorkflowRunFailureInfo failInfo =
					(ALDWorkflowRunFailureInfo)eventInfo;
			StringBuffer msg = new StringBuffer();
			
			// put some general information to the header
			msg.append("Something went wrong during operator execution!\n");
			msg.append("\nException message:\n");
			msg.append(event.getEventMessage(120) + "\n");
			msg.append("Exception class type: \n");
			msg.append(failInfo.getException().getClass() + "\n");
			msg.append("\nException stack trace:\n");
			StackTraceElement[] trace = failInfo.getException().getStackTrace(); 
			for (StackTraceElement elem : trace) {
				msg.append(elem.toString());
				msg.append("\n");
			}
			this.failureMessageWin = new JFrame("Operator Execution Failure!");
			this.failureMessageWin.setLayout(new BorderLayout());
			this.failureMessageWin.setSize(500, 300);
			JTextArea textArea = new JTextArea(msg.toString());
			textArea.setEditable(false);
			JScrollPane scrollPane = new JScrollPane(textArea);
			this.failureMessageWin.add(scrollPane,BorderLayout.CENTER);
			JPanel buttonPanel = new JPanel();
			JButton okButton = new JButton("Ok");
			okButton.setActionCommand("closeFailWin");
			okButton.addActionListener(this);
			buttonPanel.add(okButton);
			this.failureMessageWin.add(buttonPanel,BorderLayout.SOUTH);
			this.failureMessageWin.setVisible(true);
		}
	}
	
	/**
	 * Adds a node to the workflow graph.
	 * @param op		Operator associated with the node.
	 * @throws ALDOperatorException
	 */
	protected synchronized void handleAddNodeEvent(ALDOperator op,
																										ALDWorkflowNodeID id) {
		// if the window has already been initialized, ignore addition of 
		// further nodes...
		if (this.controlWin != null)
			return;

		// init the control frame
		ALDOperatorControlFrame controlFrame = null;
		try {
			controlFrame = this.getNewConfigWin(op);
			this.paramChangeListener = new ParameterChangeListener(id);
			controlFrame.addParameterChangeEventListener(this.paramChangeListener);
		} catch (ALDOperatorException e) {
			System.out.println("[ALDOperatorGUIExecutionProxy::addNode] " 
					+ "cannot instantiating control frame for \"" +op.getName()+ "\"...");
			return;
		}

		// store the control frame 
		this.controlWin = controlFrame;
		// notify workflow of new operator configuration
		// (necessary because parameter values might have changed during window
		//  setup, e.g. for images where the focussed window becomes the default)
		this.paramChangeListener.handleALDParameterChangeEvent(
																			new ALDOpParameterChangeEvent(this));
	}

	/**
	 * Update parameter labels according to configuration states.
	 * @param idList	List of nodes that are to be updated.
	 */
	protected synchronized void handleNodeParameterChangeEvent(
																			Collection<ALDWorkflowNodeID> idList) {
		// in this case list contains only a single node
		for (ALDWorkflowNodeID nodeID: idList) {
			try {
				// update the configuration window
//				System.out.println("Inputs missing....");
//				System.out.println(
//						this.alidaWorkflow.getOperator(nodeID).unconfiguredItems().size());
				// deactivate value change events
				ALDDataIOManagerSwing.getInstance().setTriggerValueChangeEvents(false);

				this.controlWin.updateOperatorAfterConfigReload(this.alidaWorkflow.getOperator(nodeID));

				// reactivate value change events
				ALDDataIOManagerSwing.getInstance().setTriggerValueChangeEvents(true);
				this.controlWin.updateParamConfigurationStatus(
									this.alidaWorkflow.getOperator(nodeID).unconfiguredItems());
			}
			catch (ALDWorkflowException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	/**
	 * Update control window color according to node's state.
	 * @param idList	List of nodes that are to be updated.
	 */
	protected synchronized void handleNodeStateChangeEvent(
																			Collection<ALDWorkflowNodeID> idList) {
		// in this case list contains only a single node
		for (ALDWorkflowNodeID nodeID: idList) {
			try {
				ALDWorkflowNodeState nodeState = this.alidaWorkflow.getState(nodeID);
				// update the configuration window
				this.controlWin.updateNodeStatus(nodeState);
			}
			catch (ALDWorkflowException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Generate a new configuration window.
	 * <p>
	 * Is to be overwritten by subclasses.
	 * 
	 * @param op		Operator for which window is requested.
	 * @return	New configuration window.
	 * @throws ALDOperatorException
	 */
	protected ALDOperatorControlFrame getNewConfigWin(ALDOperator op) 
			throws ALDOperatorException {
		return new ALDOperatorControlFrame(op, this);
	}
	
	/* *************************************************
	 * Static class functions, mainly internal helpers. 
	 * *************************************************/
	
	/**
	 * Listener class to react on parameter value changes in config window.
	 * @author moeller
	 */
	protected class ParameterChangeListener 
		implements ALDOpParameterChangeEventListener {

		/**
		 * Corresponding node ID.
		 */
		private ALDWorkflowNodeID id;
		
		/**
		 * Default constructor.
		 * @param nodeID	Alida workflow node ID of associated operator node.
		 */
		public ParameterChangeListener(ALDWorkflowNodeID nodeID) {
			this.id = nodeID;
		}
		
		/**
		 * Updates the ID of the workflow node associated with this listener.
		 * @param nodeID	New node ID.
		 */
		public void updateNodeID(ALDWorkflowNodeID nodeID) {
			this.id = nodeID;
		}
		
		@Override
    public void handleALDParameterChangeEvent(ALDOpParameterChangeEvent e) {
			try {
				// notify workflow of change in node parameters
	      ALDOperatorGUIExecutionProxy.this.alidaWorkflow.nodeParameterChanged(
	      																															this.id);
	      // process event queue
	      ALDOperatorGUIExecutionProxy.this.processWorkflowEventQueue();
      } catch (ALDWorkflowException ex) {
      	System.err.println("[ParameterChangeListener] Warning! " 
      			+ "could not propagate parameter change event, node not found!");
      }	  
    }
	}
}
